@@ -2,6 +2,7 @@
 """
     Runs the benchmarks
 """
+from __future__ import print_function
 import sys
 import os
 import re
@@ -39,14 +40,13 @@ def run_bench(name, use_gevent=False):
 
 
 def bench_wrapper(use_gevent=False):
-    print '=' * 80
-    print 'Running benchmark with Logbook %s (gevent enabled=%s)' % \
-        (version, use_gevent)
-    print '-' * 80
+    print('=' * 80)
+    print('Running benchmark with Logbook %s (gevent enabled=%s)' % (version, use_gevent))
+    print('-' * 80)
     os.chdir(bench_directory)
     for bench in list_benchmarks():
         run_bench(bench, use_gevent)
-    print '-' * 80
+    print('-' * 80)
 
 
 def main():
