@@ -11,12 +11,10 @@
 
 esc = "\x1b["
 
-codes = {}
-codes[""]          = ""
-codes["reset"]     = esc + "39;49;00m"
+codes = {"": "", "reset": esc + "39;49;00m"}
 
-dark_colors  = ["black", "darkred", "darkgreen", "brown", "darkblue",
-                "purple", "teal", "lightgray"]
+dark_colors = ["black", "darkred", "darkgreen", "brown", "darkblue",
+               "purple", "teal", "lightgray"]
 light_colors = ["darkgray", "red", "green", "yellow", "blue",
                 "fuchsia", "turquoise", "white"]
 
@@ -28,9 +26,9 @@ for d, l in zip(dark_colors, light_colors):
 
 del d, l, x
 
-codes["darkteal"]   = codes["turquoise"]
+codes["darkteal"] = codes["turquoise"]
 codes["darkyellow"] = codes["brown"]
-codes["fuscia"]     = codes["fuchsia"]
+codes["fuscia"] = codes["fuchsia"]
 
 
 def _str_to_type(obj, strtype):
@@ -42,5 +40,5 @@ def _str_to_type(obj, strtype):
 
 def colorize(color_key, text):
     """Returns an ANSI formatted text with the given color."""
-    return _str_to_type(codes[color_key], text) + text + \
-           _str_to_type(codes["reset"], text)
+    return (_str_to_type(codes[color_key], text) + text +
+            _str_to_type(codes["reset"], text))
