@@ -10,26 +10,37 @@
 """
 import re
 import os
-from collections import defaultdict
-from cgi import parse_qsl
+import platform
 
-from logbook.base import RecordDispatcher, dispatch_record, NOTSET, ERROR, NOTICE
-from logbook.handlers import Handler, StringFormatter, \
-     StringFormatterHandlerMixin, StderrHandler
+from collections import defaultdict
+from functools import partial
+
+from logbook.base import (
+    RecordDispatcher, dispatch_record, NOTSET, ERROR, NOTICE)
+from logbook.handlers import (
+    Handler, StringFormatter, StringFormatterHandlerMixin, StderrHandler)
 from logbook._termcolors import colorize
 from logbook.helpers import PY2, string_types, iteritems, u
-
 from logbook.ticketing import TicketingHandler as DatabaseHandler
 from logbook.ticketing import BackendBase
 
+try:
+    import riemann_client.client
+    import riemann_client.transport
+except ImportError:
+    riemann_client = None
+    #from riemann_client.transport import TCPTransport, UDPTransport, BlankTransport
+
+
 if PY2:
     from urllib import urlencode
+    from urlparse import parse_qsl
 else:
-    from urllib.parse import urlencode
+    from urllib.parse import parse_qsl, urlencode
 
-_ws_re = re.compile(r'(\s+)(?u)')
-TWITTER_FORMAT_STRING = \
-u('[{record.channel}] {record.level_name}: {record.message}')
+_ws_re = re.compile(r'(\s+)', re.UNICODE)
+TWITTER_FORMAT_STRING = u(
+    '[{record.channel}] {record.level_name}: {record.message}')
 TWITTER_ACCESS_TOKEN_URL = 'https://twitter.com/oauth/access_token'
 NEW_TWEET_URL = 'https://api.twitter.com/1/statuses/update.json'
 
@@ -104,8 +115,8 @@ class TaggingLogger(RecordDispatcher):
     def __init__(self, name=None, tags=None):
         RecordDispatcher.__init__(self, name)
         # create a method for each tag named
-        list(setattr(self, tag, lambda msg, *args, **kwargs:
-            self.log(tag, msg, *args, **kwargs)) for tag in (tags or ()))
+        for tag in (tags or ()):
+            setattr(self, tag, partial(self.log, tag))
 
     def log(self, tags, msg, *args, **kwargs):
         if isinstance(tags, string_types):
@@ -113,8 +124,9 @@ class TaggingLogger(RecordDispatcher):
         exc_info = kwargs.pop('exc_info', None)
         extra = kwargs.pop('extra', {})
         extra['tags'] = list(tags)
+        frame_correction = kwargs.pop('frame_correction', 0)
         return self.make_record_and_handle(NOTSET, msg, args, kwargs,
-                                           exc_info, extra)
+                                           exc_info, extra, frame_correction)
 
 
 class TaggingHandler(Handler):
@@ -207,7 +219,8 @@ class TwitterHandler(Handler, StringFormatterHandlerMixin):
     def tweet(self, status):
         """Tweets a given status.  Status must not exceed 140 chars."""
         client = self.make_client()
-        resp, content = client.request(NEW_TWEET_URL, 'POST',
+        resp, content = client.request(
+            NEW_TWEET_URL, 'POST',
             body=urlencode({'status': status.encode('utf-8')}),
             headers={'Content-Type': 'application/x-www-form-urlencoded'})
         return resp['status'] == '200'
@@ -216,6 +229,33 @@ class TwitterHandler(Handler, StringFormatterHandlerMixin):
         self.tweet(self.format(record))
 
 
+class SlackHandler(Handler, StringFormatterHandlerMixin):
+
+    """A handler that logs to slack.  Requires that you sign up an
+    application on slack and request an api token.  Furthermore the
+    slacker library has to be installed.
+    """
+
+    def __init__(self, api_token, channel, level=NOTSET, format_string=None, filter=None,
+                 bubble=False):
+
+        Handler.__init__(self, level, filter, bubble)
+        StringFormatterHandlerMixin.__init__(self, format_string)
+        self.api_token = api_token
+
+        try:
+            from slacker import Slacker
+        except ImportError:
+            raise RuntimeError('The slacker library is required for '
+                               'the SlackHandler.')
+
+        self.channel = channel
+        self.slack = Slacker(api_token)
+
+    def emit(self, record):
+        self.slack.chat.post_message(channel=self.channel, text=self.format(record))
+
+
 class JinjaFormatter(object):
     """A formatter object that makes it easy to format using a Jinja 2
     template instead of a format string.
@@ -266,11 +306,11 @@ class ExternalApplicationHandler(Handler):
         self._subprocess = subprocess
 
     def emit(self, record):
-        args = [arg.format(record=record).encode(self.encoding)
+        args = [arg.format(record=record)
                 for arg in self._arguments]
         if self._stdin_format is not None:
-            stdin_data = self._stdin_format.format(record=record) \
-                                           .encode(self.encoding)
+            stdin_data = (self._stdin_format.format(record=record)
+                          .encode(self.encoding))
             stdin = self._subprocess.PIPE
         else:
             stdin = None
@@ -284,15 +324,36 @@ class ColorizingStreamHandlerMixin(object):
     """A mixin class that does colorizing.
 
     .. versionadded:: 0.3
+    .. versionchanged:: 1.0.0
+       Added Windows support if `colorama`_ is installed.
+
+    .. _`colorama`: https://pypi.org/pypi/colorama
     """
+    _use_color = None
+
+    def force_color(self):
+        """Force colorizing the stream (`should_colorize` will return True)
+        """
+        self._use_color = True
+
+    def forbid_color(self):
+        """Forbid colorizing the stream (`should_colorize` will return False)
+        """
+        self._use_color = False
 
     def should_colorize(self, record):
         """Returns `True` if colorizing should be applied to this
         record.  The default implementation returns `True` if the
-        stream is a tty and we are not executing on windows.
+        stream is a tty. If we are executing on Windows, colorama must be
+        installed.
         """
         if os.name == 'nt':
-            return False
+            try:
+                import colorama
+            except ImportError:
+                return False
+        if self._use_color is not None:
+            return self._use_color
         isatty = getattr(self.stream, 'isatty', None)
         return isatty and isatty()
 
@@ -304,9 +365,8 @@ class ColorizingStreamHandlerMixin(object):
             return 'yellow'
         return 'lightgray'
 
-    def format_and_encode(self, record):
-        rv = super(ColorizingStreamHandlerMixin, self) \
-                .format_and_encode(record)
+    def format(self, record):
+        rv = super(ColorizingStreamHandlerMixin, self).format(record)
         if self.should_colorize(record):
             color = self.get_color(record)
             if color:
@@ -320,18 +380,35 @@ class ColorizedStderrHandler(ColorizingStreamHandlerMixin, StderrHandler):
     not colorize on Windows systems.
 
     .. versionadded:: 0.3
+    .. versionchanged:: 1.0
+       Added Windows support if `colorama`_ is installed.
+
+    .. _`colorama`: https://pypi.org/pypi/colorama
     """
+    def __init__(self, *args, **kwargs):
+        StderrHandler.__init__(self, *args, **kwargs)
+
+        # Try import colorama so that we work on Windows. colorama.init is a
+        # noop on other operating systems.
+        try:
+            import colorama
+        except ImportError:
+            pass
+        else:
+            colorama.init()
 
 
 # backwards compat.  Should go away in some future releases
-from logbook.handlers import FingersCrossedHandler as \
-     FingersCrossedHandlerBase
+from logbook.handlers import (
+    FingersCrossedHandler as FingersCrossedHandlerBase)
+
+
 class FingersCrossedHandler(FingersCrossedHandlerBase):
     def __init__(self, *args, **kwargs):
         FingersCrossedHandlerBase.__init__(self, *args, **kwargs)
         from warnings import warn
         warn(PendingDeprecationWarning('fingers crossed handler changed '
-            'location.  It\'s now a core component of Logbook.'))
+             'location.  It\'s now a core component of Logbook.'))
 
 
 class ExceptionHandler(Handler, StringFormatterHandlerMixin):
@@ -359,10 +436,12 @@ class ExceptionHandler(Handler, StringFormatterHandlerMixin):
             raise self.exc_type(self.format(record))
         return False
 
+
 class DedupHandler(Handler):
     """A handler that deduplicates log messages.
 
-    It emits each unique log record once, along with the number of times it was emitted.
+    It emits each unique log record once, along with the number of times it was
+    emitted.
     Example:::
 
         with logbook.more.DedupHandler():
@@ -375,11 +454,13 @@ class DedupHandler(Handler):
        message repeated 2 times: foo
        message repeated 1 times: bar
     """
-    def __init__(self, format_string='message repeated {count} times: {message}', *args, **kwargs):
+    def __init__(self,
+                 format_string='message repeated {count} times: {message}',
+                 *args, **kwargs):
         Handler.__init__(self, bubble=False, *args, **kwargs)
         self._format_string = format_string
         self.clear()
-        
+
     def clear(self):
         self._message_to_count = defaultdict(int)
         self._unique_ordered_records = []
@@ -392,17 +473,106 @@ class DedupHandler(Handler):
         Handler.pop_thread(self)
         self.flush()
 
+    def pop_context(self):
+        Handler.pop_context(self)
+        self.flush()
+
+    def pop_greenlet(self):
+        Handler.pop_greenlet(self)
+        self.flush()
+
     def handle(self, record):
-        if not record.message in self._message_to_count:
+        if record.message not in self._message_to_count:
             self._unique_ordered_records.append(record)
         self._message_to_count[record.message] += 1
         return True
 
     def flush(self):
         for record in self._unique_ordered_records:
-            record.message = self._format_string.format(message=record.message, count=self._message_to_count[record.message])
-            # record.dispatcher is the logger who created the message, it's sometimes supressed (by logbook.info for example)
-            dispatch = record.dispatcher.call_handlers if record.dispatcher is not None else dispatch_record
+            record.message = self._format_string.format(
+                message=record.message,
+                count=self._message_to_count[record.message])
+            # record.dispatcher is the logger who created the message,
+            # it's sometimes supressed (by logbook.info for example)
+            if record.dispatcher is not None:
+                dispatch = record.dispatcher.call_handlers
+            else:
+                dispatch = dispatch_record
             dispatch(record)
         self.clear()
 
+
+class RiemannHandler(Handler):
+
+    """
+    A handler that sends logs as events to Riemann.
+    """
+
+    def __init__(self,
+                 host,
+                 port,
+                 message_type="tcp",
+                 ttl=60,
+                 flush_threshold=10,
+                 bubble=False,
+                 filter=None,
+                 level=NOTSET):
+        """
+        :param host: riemann host
+        :param port: riemann port
+        :param message_type: selects transport. Currently available 'tcp' and 'udp'
+        :param ttl: defines time to live in riemann
+        :param flush_threshold: count of events after which we send to riemann
+        """
+        if riemann_client is None:
+            raise NotImplementedError("The Riemann handler requires the riemann_client package") # pragma: no cover
+        Handler.__init__(self, level, filter, bubble)
+        self.host = host
+        self.port = port
+        self.ttl = ttl
+        self.queue = []
+        self.flush_threshold = flush_threshold
+        if message_type == "tcp":
+            self.transport = riemann_client.transport.TCPTransport
+        elif message_type == "udp":
+            self.transport = riemann_client.transport.UDPTransport
+        elif message_type == "test":
+            self.transport = riemann_client.transport.BlankTransport
+        else:
+            msg = ("Currently supported message types for RiemannHandler are: {0}. \
+                    {1} is not supported."
+                   .format(",".join(["tcp", "udp", "test"]), message_type))
+            raise RuntimeError(msg)
+
+    def record_to_event(self, record):
+        from time import time
+        tags = ["log", record.level_name]
+        msg = str(record.exc_info[1]) if record.exc_info else record.msg
+        channel_name = str(record.channel) if record.channel else "unknown"
+        if any([record.level_name == keywords
+                for keywords in ["ERROR", "EXCEPTION"]]):
+            state = "error"
+        else:
+            state = "ok"
+        return {"metric_f": 1.0,
+                "tags": tags,
+                "description": msg,
+                "time": int(time()),
+                "ttl": self.ttl,
+                "host": platform.node(),
+                "service": "{0}.{1}".format(channel_name, os.getpid()),
+                "state": state
+                }
+
+    def _flush_events(self):
+        with riemann_client.client.QueuedClient(self.transport(self.host, self.port)) as cl:
+            for event in self.queue:
+                cl.event(**event)
+            cl.flush()
+        self.queue = []
+
+    def emit(self, record):
+        self.queue.append(self.record_to_event(record))
+
+        if len(self.queue) == self.flush_threshold:
+            self._flush_events()
